
#include "sdk_config.h"
#include "ble_light.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_l2cap.h"
#include "ble_srv_common.h"
#include "app_util.h"
#include "app_error.h"
#define NRF_LOG_MODULE_NAME "BLE_LIGHT"
#include "nrf_log.h"

/* Base UUID: 274bxxxx-b9cd-4e5e-94c4-1248b42b82f8 */
static const ble_uuid128_t m_base_uuid128 =
{
   {
       0xf8, 0x82, 0x2b, 0xb4, 0x48, 0x12, 0xc4, 0x94,
       0x5e, 0x4e, 0xcd, 0xb9, 0x00, 0x00, 0x4b, 0x27
   }
};


/**@brief Function for handling the Connect event.
 *
 * @param[in]   p_hts       Health Thermometer Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_light_t * p_light, ble_evt_t * p_ble_evt)
{
    uint32_t err_code;
    
    p_light->conn_handle = p_ble_evt->evt.gatts_evt.conn_handle;
    err_code = sd_ble_gatts_sys_attr_set(p_light->conn_handle, NULL, 0, 0);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the Disconnect event.
 *
 * @param[in]   p_light       Health Thermometer Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_light_t * p_light, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_light->conn_handle = BLE_CONN_HANDLE_INVALID;
}


/**@brief Function for handling write events to the Light ID characteristic.
 *
 * @param[in]   p_light         Light Detector Service structure.
 * @param[in]   p_evt_write   Write event received from the BLE stack.
 */
static void on_cccd_write(ble_light_t * p_light, ble_gatts_evt_write_t * p_evt_write)
{
    if (p_evt_write->len == 2)
    {
        // CCCD written, update indication state
        if (p_light->evt_handler != NULL)
        {
            ble_light_evt_t evt;

            if (ble_srv_is_indication_enabled(p_evt_write->data))
            {
                evt.evt_type = BLE_LIGHT_EVT_INDICATION_ENABLED;
            }
            else
            {
                evt.evt_type = BLE_LIGHT_EVT_INDICATION_DISABLED;
            }

            p_light->evt_handler(p_light, &evt);
        }
    }
}


/**@brief Function for handling the Write event.
 *
 * @param[in]   p_light       Light Detector Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_light_t * p_light, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if (p_evt_write->handle == p_light->meas_handles.cccd_handle)
    {
        on_cccd_write(p_light, p_evt_write);
    }
}

void ble_light_on_ble_evt(ble_light_t * p_light, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {		
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_light, p_ble_evt);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_light, p_ble_evt);
            break;

        case BLE_GATTS_EVT_WRITE:
            on_write(p_light, p_ble_evt);
            break;
	    
        default:
            // No implementation needed.
            break;
    }
}

/**@brief Function for adding Light ID characteristics.
 *
 * @param[in]   p_hts        Light Detector Service structure.
 * @param[in]   p_hts_init   Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
static uint32_t light_id_char_add(ble_light_t * p_light, const ble_light_init_t * p_light_init, ble_uuid_t *p_service_uuid)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    uint8_t             init_value[LIGHT_ID_LEN];
    
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc    = NULL;
    char_md.p_char_pf           = NULL;
    char_md.p_user_desc_md      = NULL;
    char_md.p_cccd_md           = &cccd_md;
    char_md.p_sccd_md           = NULL;

    ble_uuid.uuid = BLE_UUID_LIGHTDETECT_LIGHT_ID_CHAR;
    ble_uuid.type = p_service_uuid->type;

    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);

    memset(&attr_char_value, 0, sizeof(attr_char_value));
    memset(init_value, 0, LIGHT_ID_LEN );

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = LIGHT_ID_LEN;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = LIGHT_ID_LEN;
    attr_char_value.p_value   = init_value;

    return sd_ble_gatts_characteristic_add(p_light->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_light->meas_handles);
}

uint32_t ble_light_init(ble_light_t * p_light, const ble_light_init_t * p_light_init, ble_uuid_t *p_service_uuid)
{
    uint32_t   err_code;

    // Initialize service structure
    p_light->evt_handler = p_light_init->evt_handler;
    p_light->conn_handle = BLE_CONN_HANDLE_INVALID;

    p_service_uuid->uuid = BLE_UUID_LIGHTDETECT_SERVICE;
    err_code = sd_ble_uuid_vs_add(&m_base_uuid128, &(p_service_uuid->type));
    APP_ERROR_CHECK(err_code);

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, p_service_uuid, &p_light->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    // Add measurement characteristic
    err_code = light_id_char_add(p_light, p_light_init,p_service_uuid );
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    return NRF_SUCCESS;
}


uint32_t ble_light_measurement_send(ble_light_t * p_light, ble_light_meas_t * p_light_meas)
{
    uint32_t err_code;

    // Send value if connected
    if (p_light->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        static uint8_t                light_id_bytes[LIGHT_ID_LEN];
        ble_gatts_hvx_params_t hvx_params;
	uint16_t               hvx_len;


	light_id_bytes[0] = ( p_light_meas->light_id ) & 0xFF;
	light_id_bytes[1] = ( ( p_light_meas->light_id ) & 0xFF00 ) >> 8;
	light_id_bytes[2] = ( ( p_light_meas->light_id ) & 0xFF0000 ) >> 16;
	hvx_len = LIGHT_ID_LEN;

        memset(&hvx_params, 0, sizeof(hvx_params));

        hvx_params.handle = p_light->meas_handles.value_handle;
        hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset = 0;
        hvx_params.p_len  = &hvx_len;
        hvx_params.p_data = light_id_bytes;

        err_code = sd_ble_gatts_hvx(p_light->conn_handle, &hvx_params);
        if ((err_code == NRF_SUCCESS) && (hvx_len != LIGHT_ID_LEN))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}


uint32_t ble_light_is_indication_enabled(ble_light_t * p_light, bool * p_indication_enabled)
{
    uint32_t err_code;
    uint8_t  cccd_value_buf[BLE_CCCD_VALUE_LEN];
    ble_gatts_value_t gatts_value;

    // Initialize value struct.
    memset(&gatts_value, 0, sizeof(gatts_value));

    gatts_value.len     = BLE_CCCD_VALUE_LEN;
    gatts_value.offset  = 0;
    gatts_value.p_value = cccd_value_buf;

    err_code = sd_ble_gatts_value_get(p_light->conn_handle,
                                      p_light->meas_handles.cccd_handle,
                                      &gatts_value);
    if (err_code == NRF_SUCCESS)
    {
        *p_indication_enabled = ble_srv_is_indication_enabled(cccd_value_buf);
    }
    if (err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING)
    {
        *p_indication_enabled = false;
        return NRF_SUCCESS;
    }
    return err_code;
}
