
/** @file
 *
 * @defgroup ble_light Light detector service
 * @{
 * @brief Light detector service module.
 *
 * @details This module implements the Light Detector Serice.
 *
 *          If an event handler is supplied by the application, the Light Detector
 *          Service will generate Light Detector Service events to the application.
 *
 * @note The application must propagate BLE stack events to the Light Detector
 *       module by calling ble_hts_on_ble_evt() from the @ref softdevice_handler function.
 *
 */

#ifndef BLE_LIGHT_H__
#define BLE_LIGHT_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"
#include "ble_date_time.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Full UUID: 274b15b0-b9cd-4e5e-94c4-1248b42b82f8 */	
#define BLE_UUID_LIGHTDETECT_SERVICE	0x15B0
/* Full UUID: 274b0100-b9cd-4e5e-94c4-1248b42b82f8 */	
#define	BLE_UUID_LIGHTDETECT_LIGHT_ID_CHAR	0x0100
		
#define LIGHT_ID_LEN	3

/**@brief Light Detector Service event type. */
typedef enum
{
    BLE_LIGHT_EVT_INDICATION_ENABLED,                                         /**< Light Detector value indication enabled event. */
    BLE_LIGHT_EVT_INDICATION_DISABLED,                                        /**< Light Detector value indication disabled event. */
} ble_light_evt_type_t;

/**@brief Light Detector Service event. */
typedef struct
{
    ble_light_evt_type_t evt_type;                                            /**< Type of event. */
} ble_light_evt_t;

// Forward declaration of the ble_hts_t type.
typedef struct ble_light_s ble_light_t;

/**@brief Light Detector Service event handler type. */
typedef void (*ble_light_evt_handler_t) (ble_light_t * p_light, ble_light_evt_t * p_evt);

/**@brief Light Detector Service init structure. This contains all options and data
 *        needed for initialization of the service. */
typedef struct
{
    ble_light_evt_handler_t        evt_handler;                               /**< Event handler to be called for handling events in the Light Detector Service. */
} ble_light_init_t;

/**@brief Light Detector Service structure. This contains various status information for
 *        the service. */
struct ble_light_s
{
    ble_light_evt_handler_t        evt_handler;                               /**< Event handler to be called for handling events in the Light Detector Service. */
    uint16_t                     service_handle;                            /**< Handle of Light Detector Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t     meas_handles;                              /**< Handles related to the Light Detector Measurement characteristic. */
    uint16_t                     conn_handle;                               /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
};

/**@brief Light Detector Service measurement structure. This contains a Light Detector
 *        measurement. */
typedef struct ble_light_meas_s
{
    uint32_t                      light_id;                                 /* ID of the detected light */
} ble_light_meas_t;

/**@brief Function for initializing the Light Detector Service.
 *
 * @param[out]  p_light       Light Detector Service structure. This structure will have to
 *                          be supplied by the application. It will be initialized by this function,
 *                          and will later be used to identify this particular service instance.
 * @param[in]   p_light_init  Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
uint32_t ble_light_init(ble_light_t * p_light, const ble_light_init_t * p_light_init, ble_uuid_t *p_service_uuid);

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @details Handles all events from the BLE stack of interest to the Light Detector Service.
 *
 * @param[in]   p_light      Light Detector Service structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_light_on_ble_evt(ble_light_t * p_light, ble_evt_t * p_ble_evt);

/**@brief Function for sending light detector measurement if indication has been enabled.
 *
 * @details The application calls this function after having performed a Light Detector
 *          measurement. If indication has been enabled, the measurement data is encoded and
 *          sent to the client.
 *
 * @param[in]   p_light       Light Detector Service structure.
 * @param[in]   p_light_meas  Pointer to new light detector measurement.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t ble_light_measurement_send(ble_light_t * p_light, ble_light_meas_t * p_light_meas);

/**@brief Function for checking if indication of Light Detector is currently enabled.
 *
 * @param[in]   p_light                  Light Detector Service structure.
 * @param[out]  p_indication_enabled   TRUE if indication is enabled, FALSE otherwise.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t ble_light_is_indication_enabled(ble_light_t * p_light, bool * p_indication_enabled);


#ifdef __cplusplus
}
#endif

#endif // BLE_LIGHT_H__

/** @} */
